; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.

; ________________________________________________
; |       Programma scheda didattica 8 led       |
; |_________________PROGRAMMA 4__________________|
; |   Src File                                   |
; |______________________________________________|
; |   Tipo      : ASM                            |
; |   Versione  : 1                              |
; |   Release   : 0                              |
; |   Autore    : Lain Walter                    |
; |   Data      : 14/01/07                       |
; |______________________________________________|

;Toggle di un'uscita con timing ~245ms utilizzando TIMER e IRQ
;Pulsante di start e stop

        LIST

    __CONFIG 0x2118
; Program code protection: OFF
; Data code protection: OFF
; LVP: OFF
; BODEN: OFF
; MCLRE: Vcc
; FOSC2: INTRC
; PWRT: OFF
; WDT: OFF
; FOSC1-0: RA6,RA7 = I/O
 

#include "P16F628A.inc"
#include "macro.inc"
#include "costanti.inc"

BANK_0
#include "variabili.inc"

;***************************************************************************
;                                   RESET
;***************************************************************************

 ORG RESET_VECTOR
        GOTO SYS_STARTUP


;***************************************************************************
;                                INTERRUPTS
;***************************************************************************

 ORG INTERRUPT_VECTOR

        PUSH                            ;
        BANKSET 0                       ;Bank0
        BTFSS PIR1,TMR1IF               ;Controlla flag IRQ Timer1
        GOTO INT_END                    ; non attivo, prosegue
        CLRF PIR1                       ; clear flags IRQ
        MOVLF c_TMR_TIME_H,TMR1H        ;reimposta durata timer1
        MOVLF c_TMR_TIME_L,TMR1L        ;
        CLRF PIR1                       ;
        BANKSET 1                       ;Bank1
        BSF PIE1,TMR1IE                 ;Abilita IRQ Timer1
        BANKSET 0                       ;Bank0
        BSF f_TIMER1_ON                 ;attiva flag timer1
INT_END POP                             ;
        RETFIE


;***************************************************************************
;                             START-UP SISTEMA
;***************************************************************************
SYS_STARTUP:

        CLI                             ;Disabilita master IRQ
        CLRF PORTA                      ;Inizializza PORTA
        MOVLF 0x07,CMCON                ;disabilita comparatori
        BANKSET 1                       ;Bank1
        MOVLF 0xB,PCON                  ;Setta osc. interno 4MHz
        MOVLF 0xFD,c_LED_TRIS           ;Setta RA1 come uscita
        BANKSET 0                       ;Bank0
        BSF c_LED_PORT,c_LED_PIN        ;Spegne led
        CALL Timer_Init                 ;Inizializza i timer
        MOVLF 0xFF,b_CONT1              ;carica contatore 1
        CLRF b_SW_BUFF                  ;init registri gestione switch
        MOVLF 0x80,b_SW_CTRL            ;
        BSF INTCON,PEIE                 ;Abilita IRQ periferiche
        SEI                             ;Abilita master IRQ

;***************************************************************************
;                           PROGRAMMA PRINCIPALE
;***************************************************************************
PROGRAM:

        BTFSS f_TIMER1_ON               ;Controlla se avvenuto IRQ timer1
        GOTO PROGRAM                    ; no, torna a inizio ciclo

;da eseguire con timer1
        BCF f_TIMER1_ON                 ; si, clear stato IRQ timer
        CALL Keyb_Read                  ;legge pulsante
        BTFSS f_SW_STAT                 ;controlla stato pulsante
        GOTO M_CNT1                     ; non premuto, prosegue
        BCF f_SW_STAT                   ;
        BTFSC f_START_STOP              ;controlla stato esecuzione
        GOTO M_SST1                     ;in esecuzione
        BSF f_START_STOP                ; fermo, avvia esecuzione
        GOTO M_CNT1                     ;passa all'esecuzione
M_SST1  BCF f_START_STOP                ; ferma esecuzione
        GOTO PROGRAM                    ;torna a inizio ciclo

M_CNT1  BTFSS f_START_STOP
        GOTO PROGRAM
        DECFSZ b_CONT1                  ;decrementa contatore 1
        GOTO PROGRAM                    ; cicla finchè contatore 1 > 0

;da eseguire con divisore b_CONT1
        MOVLF 0xFF,b_CONT1              ;contatore 1 = 0, ricarica
        BTFSS c_LED_PORT,c_LED_PIN      ;controlla stato led
        GOTO LED_CLR                    ; clear
LED_SET BCF c_LED_PORT,c_LED_PIN        ; set-->clear
        GOTO PROGRAM                    ;torna a inizio ciclo
LED_CLR BSF c_LED_PORT,c_LED_PIN        ; clear-->set
        GOTO PROGRAM                    ;torna a inizio ciclo

#include "timer.inc"
#include "keyboard.inc"

        END
