/*This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

#include "leds.h"

/* Gestione dei "fusibili" di configurazione */
typedef unsigned int word;
word at 0x2007 CONFIG = _WDT_OFF & _PWRTE_OFF & _INTOSC_OSC_NOCLKOUT &
                        _MCLRE_OFF & _BOREN_OFF & _LVP_OFF & _DATA_CP_OFF & _CP_OFF;

/* Dichiarazione delle variabili globali */
__data __at (0x0020) volatile char b_CONTROLLO; //b_CONTROLLO     EQU H'20'               ;byte di controllo programma
__data __at (0x0021) volatile char b_STATO;     //b_STATO         EQU H'21'               ;byte di stato programma
__data __at (0x0022) volatile char b_SW1_BUFF;  //b_SW1_BUFF      EQU H'22'               ;buffer tasto 1
__data __at (0x0023) volatile char b_SW2_BUFF;  //b_SW2_BUFF      EQU H'23'               ;buffer tasto 2
__data __at (0x0024) volatile char b_SW_CTRL;   //b_SW_CTRL       EQU H'24'               ;byte di controllo tasti
__data __at (0x0025) volatile char b_TMR_FLAGS; //b_TMR_FLAGS     EQU H'25'               ;byte di stato timers
__data __at (0x0026) volatile char b_P5_PWM1;   //b_P5_PWM1       EQU H'26'               ;byte contatore PWM1
__data __at (0x0027) volatile char b_P5_PWM2;   //b_P5_PWM2       EQU H'27'               ;byte contatore PWM2


static unsigned char programma   = 0x00;
static unsigned char ledsSt      = 0xFF;
static unsigned char ledPos1     = 0x00;
static unsigned char dir         = 0x01;
static unsigned char Timer1Count = 0x00;

enum
{
  NONE,
  BLINK_ALL,
  SCAN_L,
  SCAN_R,
  SCAN_X,
  SCAN_Z,
  SCAN_PWM,
  MAX_PROGR
};

/* --- Vettore di interrupt --- */
static void interruptfunc(void) /* shadowregs */ interrupt 0
{
  if(TMR1IF)
  {
    TMR1IF     = 0x0;

    TMR1H      = 0xFC;  // Timer principale  CLK_INT= 1MHz
    TMR1L      = 0x17;

    TMR1IE     = 0x1;

    b_TMR_FLAGS |= 0x01;
  }
}


/* --- Implementazione funzioni --- */

/* Inizializza l'hardware */
static void InitMicro(void)
{
  /** Inizializza le porte IO usate */
  GIE        = 0x0;   // CLI                             ;Disabilita master IRQ
  PORTA      = 0x00;  // CLRF PORTA                      ;Inizializza PORTA
  CMCON      = 0x07;  // MOVLF 0x07,CMCON                ;disabilita comparatori
  PCON       = 0x0B;  // MOVLF 0xB,PCON                  ;Setta osc. interno 4MHz
  TRISA      = 0x3D;  // MOVLF 0xFD,TRISA                ;Setta RA7,RA6,RA1 come uscite
  TRISB      = 0xC0;  // MOVLF 0xC0,TRISB                ;Setta RB5,RB4,RB3,RB2,RB1,RB0 come uscite
  PORTA      = 0xFF;  // MOVLF 0xC2,PORTA                ;
  PORTB      = 0xFF;  // MOVLF 0xC2,PORTA                ;

  /** BEGIN: Timer initializer */
  TMR1ON     = 0x0;
  TMR1H      = 0xFC;  // Timer principale  CLK_INT= 1MHz
  TMR1L      = 0x17;
  PIR1       = 0x00;
  TMR1IE     = 0x01;
  T1CON      = 0x00;
  TMR1ON     = 0x01;
  /** END: Timer initializer */

  /** initialize timer counters */
  Timer1Count = 0x96;

  /** initialize interrupt flags  */
  PEIE       = 0x01;
  GIE        = 0x01;

  /** BEGIN: Timer initializer */
  b_STATO    = 0x00;
  b_SW1_BUFF = 0x00;
  b_SW2_BUFF = 0x00;
  b_SW_CTRL  = 0x00;
}


static void PortWrite(char bitMask)
{
  LED_SET(bitMask);
}

static void preStart(unsigned char maxTimes)
{
  unsigned char count1 = 0;
  unsigned char count2 = 0;

  /* Accendo tutti i led verdi ed il led rosso: si parte! */
  PWR_LED_ON;
  PortWrite(0x00);

  /** Creo un ritardo che mantiene i led accesi */
  for(count1=maxTimes;count1>0;count1--)
    for(count2=maxTimes;count2>0;count2--)
      ;

  /** Spengo TUTTI i led*/
  PWR_LED_OFF;
  PortWrite(0xFF);

  for(count1=maxTimes;count1>0;count1--)
    for(count2=maxTimes;count2>0;count2--)
      ;
  PWR_LED_ON;
  PortWrite(0x00);

  for(count1=maxTimes;count1>0;count1--)
    for(count2=maxTimes;count2>0;count2--)
      ;
}


static void Keyb_Read()
{
  #define BOUNCE_PREVENTION_MAX_VAL 0x40
  #define BOUNCE_PREVENTION_MIN_VAL 0x00

  if(!SW1_ST)
  {
    /** if b_SW1_BUFF == 0x40 do nothing*/
    if(b_SW1_BUFF < BOUNCE_PREVENTION_MAX_VAL)
    {
      b_SW1_BUFF ++;
      /** check for MaxVal needs just if buffer
         has been incremented*/
      if(b_SW1_BUFF == BOUNCE_PREVENTION_MAX_VAL)
      {
        b_SW_CTRL |= 0x04; //key_pressed =true;
        b_SW_CTRL |= 0x08;  //St_change=True;
      }
    }
  }
  else
  {
    if(b_SW1_BUFF > BOUNCE_PREVENTION_MIN_VAL/*MinVal*/)
    {
      b_SW1_BUFF--;

      if(b_SW1_BUFF == BOUNCE_PREVENTION_MIN_VAL/*MinVal*/)
      {
        b_SW_CTRL &= (~0x04); //key_pressed =true;
        b_SW_CTRL |= 0x08;    //St_change=True;
      }
    }
  }

  if(!SW2_ST)
  {
    if(b_SW2_BUFF < BOUNCE_PREVENTION_MAX_VAL)
    {
      b_SW2_BUFF++;
      if(b_SW2_BUFF == BOUNCE_PREVENTION_MAX_VAL)
      {
        b_SW_CTRL |= 0x01; //key_pressed =true;
        b_SW_CTRL |= 0x02;  //St_change=True;
      }
    }
  }
  else
  {
    if(b_SW2_BUFF > BOUNCE_PREVENTION_MIN_VAL)
    {
      b_SW2_BUFF--;

      if(b_SW2_BUFF == BOUNCE_PREVENTION_MIN_VAL)
      {
        b_SW_CTRL &= (~0x01); //key_pressed =true;
        b_SW_CTRL |= 0x02;    //St_change=True;
      }
    }
  }
}

static void Timer_Exec()
{
  if((b_SW_CTRL & 0x02)&&(b_SW_CTRL & 0x01))
  {
    b_SW_CTRL &= (~0x02);
    programma++;
    if(programma==MAX_PROGR)
      programma=NONE;

    if(programma==SCAN_L)
    {
      ledPos1 = 0x00;
      ledsSt  = 0xFE;
      LED_SET(ledsSt);
    }
    if(programma==SCAN_R)
    {
      ledPos1 = 0x00;
      ledsSt  = 0x8F;
      LED_SET(ledsSt);
    }
    if(programma==SCAN_X)
    {
     ledPos1 = 0x00;
     ledsSt  = 0x81;
     LED_SET(ledsSt);
    }
    if((programma==SCAN_Z)||
        (programma==SCAN_PWM))
    {
      ledPos1 = 0x00;
      ledsSt  = 0x01;
      dir     = 0x01;
      LED_SET(ledsSt);
    }
  }
}

static void Keyb_Run()
{
  if(b_SW_CTRL & 0x08)
  {
    b_SW_CTRL &= (~0x08);
    if(b_SW_CTRL & 0x04)
    {
      if(b_CONTROLLO & 0x01)
      {
        b_CONTROLLO &= ~0x01;
        PWR_LED_ON;
        //PortWrite(0xFF);
      }
      else
        b_CONTROLLO |= 0x01;
    }
  }
}

static void leds()
{
  switch(programma)
  {
    case BLINK_ALL:
    {
      if((PORTA & 0x02)>0)
      {
        ledsSt = 0x00;
      }
      else
      {
        ledsSt = 0xFF;
      }
    }
    break;

    case SCAN_L:
    {
      if(ledPos1==8)
        ledPos1=0;
      ledsSt = ~(1<<ledPos1);
      ledPos1++;
    }
    break;

    case SCAN_R:
    {
      if(ledPos1==8)
        ledPos1=0;
      ledsSt = ~(0x80>>ledPos1);
      ledPos1++;
    }
    break;

    case SCAN_X:
    {
      if(ledPos1==8)
        ledPos1=0;
      ledsSt = ~((1<<ledPos1)|(0x80>>ledPos1));
      ledPos1++;
    }
    break;

    case SCAN_Z:
    case SCAN_PWM:
    {
      if(ledPos1==8)
      {
        ledPos1=1;
        dir = ~dir;
      }

      if(dir==0x01)
        ledsSt = ~(0x01<<ledPos1);
      else
        ledsSt = ~(0x80>>ledPos1);

      if(programma==SCAN_PWM)
      {
        /** gestione PWM per led laterali */

      }

      ledPos1++;
    }
    break;
  }

  //Blink power led
  if((PORTA & 0x02)>0)
  {
    PWR_LED_ON;
  }
  else
  {
    PWR_LED_OFF;
  }
  PortWrite(ledsSt);
}

/** ---------------------------------
   Questo e' il programma principale!
 ------------------------------------ */
void main (void)
{
  /** Inizializza l'hardware */
  InitMicro();

  /** primi segni di vita */
  preStart(200);

  /** Inizio il lampeggio con il timer */
  programma = BLINK_ALL;

  /** Esegui per sempre ... */
  while (1)
  {
    if(b_TMR_FLAGS & 0x01)
    {
      b_TMR_FLAGS &= ~0x01;  //clear stato IRQ
      Timer1Count--;

      /** controlla SW1*/
      Keyb_Read();

      /** controlla SW2*/
      Keyb_Run();

      if((b_CONTROLLO & 0x01)&&(Timer1Count==0))
      {
        Timer1Count = 0x96;

        Timer_Exec();
        leds();
      }
    }
  }
}

